from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase
from django.urls import reverse

from ..middleware import ReferrerPolicyMiddleware


class ReferrerPolicyMiddlewareTests(TestCase):
    """
    Test ReferrerPolicyMiddleware.

    """
    test_url = reverse('test-referrer-policy-middleware')

    def test_valid_values(self):
        """
        Each valid value of the header is accepted, and properly sent.

        """
        for value in ReferrerPolicyMiddleware.VALID_REFERRER_POLICIES:
            with self.settings(REFERRER_POLICY=value):
                response = self.client.get(self.test_url)
                self.assertTrue('Referrer-Policy' in response)
                self.assertEqual(
                    response['Referrer-Policy'], value
                )

    def test_invalid_value(self):
        """
        An invalid header value is a configuration error.

        """
        with self.assertRaises(ImproperlyConfigured):
            with self.settings(REFERRER_POLICY='invalid'):
                self.client.get(self.test_url)

    def test_missing_value(self):
        """
        A missing header value is a configuration error.

        """
        old_value = getattr(settings, 'REFERRER_POLICY', None)
        try:
            if old_value is not None:
                delattr(settings, 'REFERRER_POLICY')
            with self.assertRaises(ImproperlyConfigured):
                self.client.get(self.test_url)
        finally:
            if old_value is not None:
                setattr(settings, 'REFERRER_POLICY', old_value)



from django.conf.urls import url
from django.http import HttpResponse


def view(request):
    """
    A minimal view for use in testing.

    """
    return HttpResponse('Content.')


urlpatterns = [
    url(r'^referrer-policy-middleware$',
        view,
        name='test-referrer-policy-middleware'),
]



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


class ReferrerPolicyMiddleware(object):
    """
    A middleware implementing the Referrer-Policy header.

    The value of the header will be read from the REFERRER_POLICY
    setting, which must be present and must be set to one of the
    string values contained in the attribute VALID_REFERRER_POLICIES
    on this class.

    """
    VALID_REFERRER_POLICIES = [
        'no-referrer',
        'no-referrer-when-downgrade',
        'origin',
        'origin-when-cross-origin',
        'same-origin',
        'strict-origin',
        'strict-origin-when-cross-origin',
        'unsafe-url',
    ]

    def __init__(self, get_response):
        self.get_response = get_response
        if not hasattr(settings, 'REFERRER_POLICY') or \
           settings.REFERRER_POLICY not in self.VALID_REFERRER_POLICIES:
            raise ImproperlyConfigured(
                "settings.REFERRER_POLICY is not set or has an illegal value."
            )

    def __call__(self, request):
        response = self.get_response(request)
        response['Referrer-Policy'] = settings.REFERRER_POLICY
        return response



"""
A standalone test runner script, configuring the minimum settings
required for tests to execute.

Re-use at your own risk: many Django applications will require
different settings and/or templates to run their tests.

"""

import os
import sys


# Make sure the app is (at least temporarily) on the import path.
APP_DIR = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, APP_DIR)


# Minimum settings required for the app's tests.
SETTINGS_DICT = {
    'BASE_DIR': APP_DIR,
    'INSTALLED_APPS': (
        'django_referrer_policy',
    ),
    'ROOT_URLCONF': 'django_referrer_policy.tests.urls',
    'DATABASES': {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': os.path.join(APP_DIR, 'db.sqlite3'),
        },
    },
    'MIDDLEWARE': (
        'django.middleware.common.CommonMiddleware',
        'django_referrer_policy.middleware.ReferrerPolicyMiddleware',
    ),
}


def run_tests():
    # Making Django run this way is a two-step process. First, call
    # settings.configure() to give Django settings to work with:
    from django.conf import settings
    settings.configure(**SETTINGS_DICT)

    # Then, call django.setup() to initialize the application cache
    # and other bits:
    import django
    django.setup()

    # Now we instantiate a test runner...
    from django.test.utils import get_runner
    TestRunner = get_runner(settings)

    # And then we run tests and return the results.
    test_runner = TestRunner(verbosity=2, interactive=True)
    failures = test_runner.run_tests(
        ['django_referrer_policy.tests']
    )
    sys.exit(bool(failures))


if __name__ == '__main__':
    run_tests()



import os

from setuptools import setup


setup(name='django-referrer-policy',
      version='1.0',
      zip_safe=False, # eggs are the devil.
      description='Referrer-Policy header implementation for Django.',
      long_description=open(os.path.join(os.path.dirname(__file__),
                                         'README.rst')).read(),
      author='James Bennett',
      author_email='james@b-list.org',
      url='https://github.com/ubernostrum/django-referrer-policy/',
      packages=['django_referrer_policy', 'django_referrer_policy.tests'],
      test_suite='django_referrer_policy.runtests.run_tests',
      classifiers=['Development Status :: 5 - Production/Stable',
                   'Environment :: Web Environment',
                   'Framework :: Django',
                   'Framework :: Django :: 1.11',
                   'Framework :: Django :: 2.0',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: BSD License',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Programming Language :: Python :: 2',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3',
                   'Programming Language :: Python :: 3.4',
                   'Programming Language :: Python :: 3.5',
                   'Programming Language :: Python :: 3.6',
                   'Topic :: Utilities'],
      install_requires=[
          'Django>=1.11',
      ]
)
